@@ -2,18 +2,13 @@
 
 import sys
 
-from pulp import *
+from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize, LpAffineExpression
 
 try:
     import path
 except ImportError:
     pass
-        
-try:
-    import path
-except ImportError:
-    pass
-        
+                
 try:
     import dippy
 except ImportError:
@@ -43,7 +38,7 @@ try:
 except ImportError:
     ASSIGNMENT_COSTS = dict((i, 0) for i in ASSIGNMENTS)
 
-display_mode = 'off'
+display_mode = 'xdot'
 
 prob = dippy.DipProblem("Facility Location", display_mode = display_mode,
                         layout = 'dot', display_interval = 0)
@@ -72,7 +67,7 @@ for i in LOCATIONS:
 for i, j in ASSIGNMENTS:
     prob.relaxation[i] += assign_vars[(i, j)] <= use_vars[i]
 
-def solve_subproblem(prob, key, redCosts):
+def solve_subproblem(prob, key, redCosts, target):
     if debug_print:
         print "solve_subproblem..."
         print redCosts
@@ -94,35 +89,24 @@ def solve_subproblem(prob, key, redCosts):
         print obj
         print "z, solution =", z, solution
         print "redCosts[use_vars[loc]] =", redCosts[use_vars[loc]]
+        print "Fixed cost, rc", FIXED_COST[loc], redCosts[use_vars[loc]] - z
 
-    rc = redCosts[use_vars[loc]] - z
-
-    if debug_print:
-        print "Fixed cost, rc", FIXED_COST[loc], rc
-
-    if rc > tol: # ... or an empty location is "useful"
-       
-        var_values = {}
-
-        var_tuple = (0.0, 0.0, var_values)
+    if redCosts[use_vars[loc]] > z + tol: # ... or an empty location is "useful"
         if debug_print:
             print "Zero solution is optimal"
-            print var_tuple
-        return [var_tuple]
+        return [{}]
 
     var_values = dict([(avars[i], 1) for i in solution])
     var_values[use_vars[loc]] = 1
 
-    cost = FIXED_COST[loc] + sum([ASSIGNMENT_COSTS[(loc, PRODUCTS[j])] for j in solution])    
-    var_tuple = (cost, rc, var_values)
-    rcCheck = 0.0
-    for v in var_values.keys():
-        rcCheck += redCosts[v] * var_values[v]
     if debug_print:
-        print "Checking rc calc", rc, rcCheck 
-        print var_tuple
-    return [var_tuple]
+        rcCheck = 0.0
+        for v in var_values.keys():
+            rcCheck += redCosts[v] * var_values[v]
+        print "Checking rc calc", redCosts[use_vars[loc]] - z, rcCheck 
+        print var_values
 
+    return [var_values]
 
 def knapsack01(obj, weights, capacity):
     """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
@@ -135,10 +119,11 @@ def knapsack01(obj, weights, capacity):
         return 0, []
 
     if (debug_subproblem):
-        relaxation = pulp.LpProblem('relaxation', pulp.LpMaximize)
+        relaxation = LpProblem('relaxation', LpMaximize)
         relax_vars = [str(i) for i in range(n)]
         var_dict   = LpVariable.dicts("", relax_vars, 0, 1, LpBinary)
-        relaxation += lpSum(var_dict[str(i)] * weights[i] for i in range(n)) <= capacity
+        relaxation += (lpSum(var_dict[str(i)] * weights[i] for i in range(n)) 
+                       <= capacity)
         relaxation += lpSum(var_dict[str(i)] * obj[i] for i in range(n))
         relaxation.solve()
         relax_obj = value(relaxation.objective)
@@ -374,10 +359,9 @@ if debug_print_lp:
     for n, i in enumerate(LOCATIONS):
         prob.writeRelaxed(n, 'facility_relax%s.lp' % i);
 
-prob.writeFull('facility.lp', 'facility.dec')
+#prob.writeFull('facility.lp', 'facility.dec')
 
-#prob.branch_method = choose_antisymmetry_branch
-prob.relaxed_solver = solve_subproblem
+#prob.relaxed_solver = solve_subproblem
 #prob.init_vars = init_one_each
 #prob.init_vars = init_first_fit
 #prob.generate_cuts = generate_weight_cuts
@@ -385,17 +369,21 @@ prob.relaxed_solver = solve_subproblem
 #prob.root_heuristic = True
 #prob.node_heuristic = True
 
-dippy.Solve(prob, {
-    'TolZero': '%s' % tol,
-    'doPriceCut': '1',
-    'CutCGL': '1',
-#    'SolveMasterAsIp': '0',
-#    'generateInitVars': '1',
-#    'LogDebugLevel': 3,
-#    'LogDumpModel': 5,
-#    'ALPS' :
-#    {'msgLevel' : 3}
-})
+dippyOpts = {}
+algo = 'PriceCut'
+if len(sys.argv) > 1:
+    algo = sys.argv[1]
+if algo == 'PriceCut':
+    dippyOpts['doPriceCut'] = '1'
+    dippyOpts['CutCGL'] = '1'
+elif algo == 'Price':
+    dippyOpts['doPriceCut'] = '1'
+    dippyOpts['CutCGL'] = '0'
+else:
+    dippyOpts['doCut'] = '1'
+dippyOpts['TolZero'] = '%s' % tol
+        
+dippy.Solve(prob, dippyOpts)
 
 if prob.display_mode != 'off':
     numNodes = len(prob.Tree.get_node_list())
