@@ -11,12 +11,14 @@ from dipapi import DipAPI
 
 from _dippy import *
 
+gimpy_installed = True
 try:
-    from coinor.gimpy import BinaryTree
+    from src.gimpy import BinaryTree
 except ImportError:
-    gimpy_installed = False
-else:
-    gimpy_installed = True
+    try:
+        from coinor.gimpy import BinaryTree
+    except ImportError:
+        gimpy_installed = False
 
 if gimpy_installed:
     try:
@@ -787,7 +789,7 @@ class DipProblem(pulp.LpProblem, DipAPI):
             errorStr = "Error in postProcessBranch\n%s" % ex
             raise DipError(errorStr)
 
-    def solveRelaxed(self, key, redCostX):
+    def solveRelaxed(self, key, redCostX, target):
         """
         Returns solutions to the whichBlock relaxed subproblem
 
@@ -812,11 +814,21 @@ class DipProblem(pulp.LpProblem, DipAPI):
                 raise DipError("Reduced cost and variable list don't match in",
                                "solveRelaxed")
     
-            dvs = self.relaxed_solver(self, key, redCostDict)
+            dvs = self.relaxed_solver(self, key, redCostDict, target)
             
             if dvs is not None:
                 if len(dvs) > 0:
-                    return dvs
+                    dvs_with_costs = []
+                    for var in dvs:
+                        if isinstance(var, dict):
+                            cost = sum(self.objective[i]*var[i] for i in var
+                                       if i in self.objective)
+                            red_cost = sum(redCostDict[i]*var[i] for i in var
+                                           if i in redCostDict)
+                            dvs_with_costs.append((cost, red_cost, var))
+                        else:
+                            return dvs
+                    return dvs_with_costs
                 else:
                     print "Empty variable list in solveRelaxed, returning None"
   
